/*
 * Appcelerator Sync Server Example Application
 *
 * This application demonstrates how to use the Sync Server Client module to
 * synchronize MS Dynamics CRM data with the Sync Server.
 *
 * Before running this application, you must have an active Sync Server instance
 * with the msDynamicsCrm1.0 adapter configured and running.
 *
 * The Sync Server Client requires a `syncConfig.json` file with the appropriate
 * configuration options defined. The `syncConfig.json` file provided with this
 * example specifies a default `appId` and `serverAddress`. You must change the values
 * of these properties to match your Sync Server's configuration. You can either change
 * these properties in the `syncConfig.json` file OR set these properties on the
 * connector when you call `createSyncConnector`.
 *
 * The Sync Server requires a user and password for authentication. You must
 * update the credentials below before authenticating with the Sync Server
 *
 * To run the example:
 *   1. Install the appcelerator.syncserver.client module (if not already installed)
 *   2. Create a new Titanium application
 *   3. Add the appcelerator.syncserver.client module to your application's tiapp.xml file
 *   4. Copy the app.js and syncConfig.json file from the example folder into your new Titanium application
 *   5. Update the appId and serverAddress properties in the syncConfig.json file
 *   6. Update the user and password credentials in the app.js file
 *   7. Run the application
 *   8. Click on 'Login' to verify that the credentials are correct
 *   9. Click on 'Logout' to verify that you are logged out
 *  10. Click on 'Sync' to pull the current records from the Sync Server
 *  11. Click on 'Queue changes' to update the local database and queue updates
 *  12. Click on 'Sync' to push and pull the records
 *
 * You can click on 'Queue changes' and 'Sync' multiple times to show the updating, deleting, and creating of
 * the sample contact record
 *
 * Progress messages are output to the console
 */

var credentials = {
	user: 'user@company.com',
	password: 'password'
}

// Create the UI for the application
createUI();

// Load the sync client module
var syncClient = require('appcelerator.syncserver.client');

// Create a connector for the msDynamicsCrm1.0 connector
var connector = syncClient.createSyncConnector({
	name: 'msDynamicsCrm1.0'
});

// Create the database -- it must exist prior to synchronization
createDatabase();

// This is an example of modifying the query parameters at runtime. Setting the dataCallback
// requires a function to be passed to it, so it must be set at runtime.
var m = connector.getModelDefinition('contact');
if (m) {
	m.syncOptions.queryParameters.statecode = 0;

	// This is an example of using a dataCallback to process the pulled records prior to updating
	// the database.
	m.syncOptions.dataCallback = function (modelDef, records, fullReplace) {
		var i, len;

		// Look for the record that we are updating in our example to show the values
		// that were pulled from the Sync Server
		for (i=0, len=records.length; i < len; i++) {
			if (records[i].contactid === '5c34e1e4-cfd2-e211-9dba-b4b52f67e641') {
				Ti.API.warn(formatJSON(records[i]));
			}
		}

		// Forward the records to the connector so it can update the database
		connector.batchUpdate(modelDef, records, fullReplace);
	}
} else {
	Ti.API.error('Unable to locate model definition');
}

//=====================================================================

function createUI() {
	var win = Ti.UI.createWindow({
		backgroundColor: 'white',
		layout: 'vertical'
	});

	var btnLogin = Ti.UI.createButton({ title: 'Login', top: 4,	left: 4, right: 4 });
	var btnLogout = Ti.UI.createButton({ title: 'Logout', top: 4,	left: 4, right: 4 });
	var btnSync = Ti.UI.createButton({ title: 'Sync', top: 4,	left: 4, right: 4 });
	var btnQueue = Ti.UI.createButton({ title: 'Queue changes', top:4, left: 4, right: 4});
	var btnCancel = Ti.UI.createButton({ title: 'Cancel', top: 4,	left: 4, right: 4 });

	btnLogin.addEventListener('click', login);
	btnLogout.addEventListener('click', logout);
	btnSync.addEventListener('click', synchronize);
	btnQueue.addEventListener('click', queueChanges);
	btnCancel.addEventListener('click', cancel);

	win.add(btnLogin);
	win.add(btnLogout);
	win.add(btnSync);
	win.add(btnQueue);
	win.add(btnCancel);

	win.open();
}

function createDatabase() {
	// Create the application's database -- make sure every non-picklist table has the 'modifiedon' field
	var db = Ti.Database.open(connector.databaseName);
	db.execute('CREATE TABLE IF NOT EXISTS contact (contactid TEXT PRIMARY KEY, fullname TEXT, firstname TEXT, lastname TEXT, modifiedon TEXT)');
	db.execute('CREATE TABLE IF NOT EXISTS opportunity_opportunityratingcode (value INTEGER PRIMARY KEY, label TEXT)');
	// For the example application, delete all existing rows from the tables so we start
	// with a clean slate every time the application is run.
	db.execute('DELETE FROM contact');
	db.execute('DELETE FROM opportunity_opportunityratingcode');
	db.close();
}

function formatJSON(evt) {
	return JSON.stringify(evt, function(key, value) {
		if(key === 'source' || key === 'type') {
        	return undefined;
    	} else {
        	return value;
    	}
	}, 2);
}

//=====================================================================

function login() {
	connector.login({
		user: credentials.user,
		password: credentials.password,
		success: function(e) {
			Ti.API.info('Successfully authenticated ' + connector.currentUser);
			Ti.API.info('SessionExpires: ' + e.sessionExpires);
		},
		error: function(e) {
			Ti.API.error('Error logging in: ' + e.message);
		},
		progress: function(e) {
			Ti.API.info("Login Progress: " + formatJSON(e));
		}
	});
}

function logout() {
	connector.logout({
		success: function (e) {
			Ti.API.info('Logged out');
		},
		error: function(e) {
			Ti.API.error('Error logging out: ' + e.message);
		},
		progress: function(e) {
			Ti.API.info("Logout Progress: " + formatJSON(e));
		}
	});
}

function queueChanges() {
	// Every record needs a unique identifier. This example application uses a
	// hard-coded GUID for demonstration purposes only. Normally your application
	// would generate a GUID for all new records that it creates.
	var id = '5c34e1e4-cfd2-e211-9dba-b4b52f67e641';
	var db = Ti.Database.open(connector.databaseName);
    try {
	    // For demonstration purposes:
	    //   If the record does not exist, create it
	    //   If the record does exist and it has its initial value, update it
	    //   Otherwise, delete it
        var cursor = db.execute('SELECT * FROM contact WHERE contactid = ?;', id);
        if (!cursor.isValidRow()) {
	        Ti.API.info('Creating new contact record');
	        db.execute('INSERT INTO contact (firstname, lastname, contactid) VALUES (?,?,?);','Peter', 'Parker', id);
	        connector.queuePendingChanges({
		        modelName: 'contact',
		        modelId: id,
		        action: syncClient.ACTION.CREATE
	        });
        } else {
	        // The record exists in our local database
	        if (cursor.fieldByName('lastname') === 'Parker') {
		        Ti.API.info('Updating existing contact record');
		        db.execute('UPDATE contact SET firstname = "Peter", lastname = "Pan" WHERE contactid = ?;', id);
		        connector.queuePendingChanges({
			        modelName: 'contact',
			        modelId: id,
			        action: syncClient.ACTION.UPDATE,
			        fields: [ 'firstname', 'lastname' ]
		        });
	        } else {
		        Ti.API.info('Deleting existing contact record');
		        db.execute('DELETE FROM contact WHERE contactid = ?;', id);
		        connector.queuePendingChanges({
			        modelName: 'contact',
			        modelId: id,
			        action: syncClient.ACTION.DELETE
		        });
	        }
        }
    } finally {
        db.close();
    }

	Ti.API.info("Num: " + connector.getNumPendingChanges());
}

function synchronize() {
	connector.sync({
		user: credentials.user,
		password: credentials.password,
		success: function (e) {
			Ti.API.info("Successfully synchronized: " + formatJSON(e));
		},
		error: function (e) {
			Ti.API.error("Error during sync: " + formatJSON(e));
		},
		progress: function(e) {
			Ti.API.info("Sync Progress: " + formatJSON(e));
		}
	});
}

function cancel() {
	connector.cancel();
}



